@@ -1,4 +1,3 @@
-/* $Id: ares_getnameinfo.c,v 1.36 2009-11-09 12:56:11 yangtse Exp $ */
 
 /* Copyright 2005 by Dominick Meglio
  *
@@ -23,9 +22,6 @@
 #  endif
 #endif
 
-#ifdef HAVE_SYS_SOCKET_H
-#  include <sys/socket.h>
-#endif
 #ifdef HAVE_NETINET_IN_H
 #  include <netinet/in.h>
 #endif
@@ -48,17 +44,9 @@
 #include <net/if.h>
 #endif
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
 #include "ares.h"
 #include "ares_ipv6.h"
-#include "inet_ntop.h"
+#include "ares_nowarn.h"
 #include "ares_private.h"
 
 struct nameinfo_query {
@@ -74,12 +62,15 @@ struct nameinfo_query {
 };
 
 #ifdef HAVE_SOCKADDR_IN6_SIN6_SCOPE_ID
-#define IPBUFSIZ 40+IF_NAMESIZE
+#define IPBUFSIZ \
+        (sizeof("ffff:ffff:ffff:ffff:ffff:ffff:255.255.255.255") + IF_NAMESIZE)
 #else
-#define IPBUFSIZ 40
+#define IPBUFSIZ \
+        (sizeof("ffff:ffff:ffff:ffff:ffff:ffff:255.255.255.255"))
 #endif
 
-static void nameinfo_callback(void *arg, int status, int timeouts, struct hostent *host);
+static void nameinfo_callback(void *arg, int status, int timeouts,
+                              struct hostent *host);
 static char *lookup_service(unsigned short port, int flags,
                             char *buf, size_t buflen);
 #ifdef HAVE_SOCKADDR_IN6_SIN6_SCOPE_ID
@@ -97,13 +88,15 @@ void ares_getnameinfo(ares_channel channel, const struct sockaddr *sa,
   struct nameinfo_query *niquery;
   unsigned int port = 0;
 
-  /* Verify the buffer size */
-  if (salen == sizeof(struct sockaddr_in))
+  /* Validate socket address family and length */
+  if ((sa->sa_family == AF_INET) &&
+      (salen == sizeof(struct sockaddr_in)))
     {
       addr = (struct sockaddr_in *)sa;
       port = addr->sin_port;
     }
-  else if (salen == sizeof(struct sockaddr_in6))
+  else if ((sa->sa_family == AF_INET6) &&
+           (salen == sizeof(struct sockaddr_in6)))
     {
       addr6 = (struct sockaddr_in6 *)sa;
       port = addr6->sin6_port;
@@ -183,22 +176,25 @@ void ares_getnameinfo(ares_channel channel, const struct sockaddr *sa,
         if (sa->sa_family == AF_INET)
           {
             niquery->family = AF_INET;
-            memcpy(&niquery->addr.addr4, addr, sizeof(addr));
-            ares_gethostbyaddr(channel, &addr->sin_addr, sizeof(struct in_addr), AF_INET,
+            memcpy(&niquery->addr.addr4, addr, sizeof(niquery->addr.addr4));
+            ares_gethostbyaddr(channel, &addr->sin_addr,
+                               sizeof(struct in_addr), AF_INET,
                                nameinfo_callback, niquery);
           }
         else
           {
             niquery->family = AF_INET6;
-            memcpy(&niquery->addr.addr6, addr6, sizeof(addr6));
-            ares_gethostbyaddr(channel, &addr6->sin6_addr, sizeof(struct in6_addr), AF_INET6,
+            memcpy(&niquery->addr.addr6, addr6, sizeof(niquery->addr.addr6));
+            ares_gethostbyaddr(channel, &addr6->sin6_addr,
+                               sizeof(struct ares_in6_addr), AF_INET6,
                                nameinfo_callback, niquery);
           }
       }
     }
 }
 
-static void nameinfo_callback(void *arg, int status, int timeouts, struct hostent *host)
+static void nameinfo_callback(void *arg, int status, int timeouts,
+                              struct hostent *host)
 {
   struct nameinfo_query *niquery = (struct nameinfo_query *) arg;
   char srvbuf[33];
@@ -217,8 +213,8 @@ static void nameinfo_callback(void *arg, int status, int timeouts, struct hosten
             service = lookup_service(niquery->addr.addr6.sin6_port,
                                      niquery->flags, srvbuf, sizeof(srvbuf));
         }
-      /* NOFQDN means we have to strip off the domain name portion.
-         We do this by determining our own domain name, then searching the string
+      /* NOFQDN means we have to strip off the domain name portion.  We do
+         this by determining our own domain name, then searching the string
          for this domain name and removing it.
        */
 #ifdef HAVE_GETHOSTNAME
@@ -227,7 +223,7 @@ static void nameinfo_callback(void *arg, int status, int timeouts, struct hosten
            char buf[255];
            char *domain;
            gethostname(buf, 255);
-           if ((domain = strchr(buf, '.')))
+           if ((domain = strchr(buf, '.')) != NULL)
              {
                char *end = ares_striendstr(host->h_name, domain);
                if (end)
@@ -235,8 +231,10 @@ static void nameinfo_callback(void *arg, int status, int timeouts, struct hosten
              }
         }
 #endif
-      niquery->callback(niquery->arg, ARES_SUCCESS, niquery->timeouts, (char *)(host->h_name),
+      niquery->callback(niquery->arg, ARES_SUCCESS, niquery->timeouts,
+                        (char *)(host->h_name),
                         service);
+      free(niquery);
       return;
     }
   /* We couldn't find the host, but it's OK, we can use the IP */
@@ -244,12 +242,15 @@ static void nameinfo_callback(void *arg, int status, int timeouts, struct hosten
     {
       char ipbuf[IPBUFSIZ];
       if (niquery->family == AF_INET)
-        ares_inet_ntop(AF_INET, &niquery->addr.addr4.sin_addr, ipbuf, IPBUFSIZ);
+        ares_inet_ntop(AF_INET, &niquery->addr.addr4.sin_addr, ipbuf,
+                       IPBUFSIZ);
       else
         {
-          ares_inet_ntop(AF_INET6, &niquery->addr.addr6.sin6_addr, ipbuf, IPBUFSIZ);
+          ares_inet_ntop(AF_INET6, &niquery->addr.addr6.sin6_addr, ipbuf,
+                         IPBUFSIZ);
 #ifdef HAVE_SOCKADDR_IN6_SIN6_SCOPE_ID
-          append_scopeid(&niquery->addr.addr6, niquery->flags, ipbuf, sizeof(ipbuf));
+          append_scopeid(&niquery->addr.addr6, niquery->flags, ipbuf,
+                         sizeof(ipbuf));
 #endif
         }
       /* They want a service too */
@@ -262,7 +263,9 @@ static void nameinfo_callback(void *arg, int status, int timeouts, struct hosten
             service = lookup_service(niquery->addr.addr6.sin6_port,
                                      niquery->flags, srvbuf, sizeof(srvbuf));
         }
-      niquery->callback(niquery->arg, ARES_SUCCESS, niquery->timeouts, ipbuf, service);
+      niquery->callback(niquery->arg, ARES_SUCCESS, niquery->timeouts, ipbuf,
+                        service);
+      free(niquery);
       return;
     }
   niquery->callback(niquery->arg, status, niquery->timeouts, NULL, NULL);
@@ -297,10 +300,12 @@ static char *lookup_service(unsigned short port, int flags,
           sep = &se;
           memset(tmpbuf, 0, sizeof(tmpbuf));
 #if GETSERVBYPORT_R_ARGS == 6
-          if (getservbyport_r(port, proto, &se, (void *)tmpbuf, sizeof(tmpbuf), &sep) != 0)
+          if (getservbyport_r(port, proto, &se, (void *)tmpbuf,
+                              sizeof(tmpbuf), &sep) != 0)
             sep = NULL;
 #elif GETSERVBYPORT_R_ARGS == 5
-          sep = getservbyport_r(port, proto, &se, (void *)tmpbuf, sizeof(tmpbuf));
+          sep = getservbyport_r(port, proto, &se, (void *)tmpbuf,
+                                sizeof(tmpbuf));
 #elif GETSERVBYPORT_R_ARGS == 4
           if (getservbyport_r(port, proto, &se, (void *)tmpbuf) != 0)
             sep = NULL;
@@ -342,11 +347,12 @@ static void append_scopeid(struct sockaddr_in6 *addr6, unsigned int flags,
 #ifdef HAVE_IF_INDEXTONAME
   int is_ll, is_mcll;
 #endif
-  char fmt_u[] = "%u";
-  char fmt_lu[] = "%lu";
+  static const char fmt_u[] = "%u";
+  static const char fmt_lu[] = "%lu";
   char tmpbuf[IF_NAMESIZE + 2];
   size_t bufl;
-  char *fmt = (sizeof(addr6->sin6_scope_id) > sizeof(unsigned int))?fmt_lu:fmt_u;
+  const char *fmt = (sizeof(addr6->sin6_scope_id) > sizeof(unsigned int))?
+    fmt_lu:fmt_u;
 
   tmpbuf[0] = '%';
 
@@ -393,8 +399,8 @@ static char *ares_striendstr(const char *s1, const char *s2)
   c2 = s2;
   while (c2 < s2+s2_len)
     {
-      lo1 = tolower(*c1);
-      lo2 = tolower(*c2);
+      lo1 = TOLOWER(*c1);
+      lo2 = TOLOWER(*c2);
       if (lo1 != lo2)
         return NULL;
       else
